<?php

class MSPS_Custom_Wallet_Manager {

	/**
	 * @var string
	 */
	protected static $hierarchy_wallet_id = 'hierarchy_point';

	/**
	 * @return void
	 */
	static function init() {
		add_filter( 'msps_wallet_items', array( __CLASS__, 'register_custom_wallet_item' ) );
		add_filter( 'msre_hierarchy_wallet_id', array( __CLASS__, 'change_hierarchy_wallet_id' ) );

		add_filter( 'msps_batch_action_settings', array( __CLASS__, 'add_hierarchy_point_batch_action_settings' ) );
		add_filter( 'msps_manage_point_settings', array( __CLASS__, 'add_manage_hierarchy_point_settings' ) );
		add_filter( 'msps_get_user_point_info', array( __CLASS__, 'add_hierarchy_point_info' ), 10, 2 );

		add_action( 'wp_ajax_msps_adjust_hierarchy_point', array( __CLASS__, 'adjust_hierarchy_point' ) );
		add_action( 'wp_ajax_msps_batch_adjust_hierarchy_point', array( __CLASS__, 'batch_adjust_hierarchy_point' ) );
	}

	/**
	 * @param $wallet_items
	 *
	 * @return mixed
	 */
	public static function register_custom_wallet_item( $wallet_items ) {
		require_once( 'wallet/class-msps-wallet-item-hierarchy-point.php' );

		$wallet_items[] = 'MSPS_Wallet_Item_Hierarchy_Point';

		return $wallet_items;
	}

	/**
	 * @param $wallet_id
	 *
	 * @return string
	 */
	public static function change_hierarchy_wallet_id( $wallet_id ) {
		return self::$hierarchy_wallet_id;
	}

	/**
	 * @param $settings
	 *
	 * @return mixed
	 */
	public static function add_hierarchy_point_batch_action_settings( $settings ) {
		$settings[] = array(
			"id"           => "batch_adjust_hierarchy_point",
			"title"        => __( "다단계포인트 일괄처리", 'mshop-point-ex' ),
			"className"    => "two wide column",
			"type"         => "MShopPointBatchAdjuster",
			"filter"       => "mshop_manage_point_filter",
			'custom_event' => 'mshop_manage_point',
			'action'       => 'msps_batch_adjust_hierarchy_point'
		);

		return $settings;
	}

	/**
	 * @param $settings
	 *
	 * @return array|string[][]|void[][]
	 */
	public static function add_manage_hierarchy_point_settings( $settings ) {
		return array_merge(
			array_slice( $settings, 0, count( $settings ) - 1 ),
			array(
				array(
					"id"           => self::$hierarchy_wallet_id,
					"title"        => __( "다단계포인트", 'mshop-point-ex' ),
					"className"    => "two wide column",
					"type"         => "MShopPointAdjuster",
					'custom_event' => 'mshop_manage_point',
					'action'       => 'msps_adjust_hierarchy_point'
				)
			),
			array_slice( $settings, count( $settings ) - 1, 1 ) );
	}

	/**
	 * @param $info
	 * @param $user
	 *
	 * @return mixed
	 */
	public static function add_hierarchy_point_info( $info, $user ) {
		$msps_user       = new MSPS_User( $user->ID );
		$hierarchy_point = $msps_user->get_point( array( self::$hierarchy_wallet_id ) );

		$info[ self::$hierarchy_wallet_id ] = array(
			'point'      => $hierarchy_point,
			'point_desc' => number_format( $hierarchy_point, wc_get_price_decimals() ),
			'id'         => $user->ID,
			'name'       => $user->data->display_name
		);

		return $info;
	}

	/**
	 * @return void
	 */
	static function adjust_hierarchy_point() {
		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			wp_send_json_error( __( '사용 권한이 없습니다.', 'mshop-point-ex' ) );
		}

		$current_language = apply_filters( 'msps_get_current_language', mshop_wpml_get_current_language() );

		$user   = new MSPS_User( $_REQUEST['id'] );
		$action = $_REQUEST['point_action'];
		$amount = $_REQUEST['amount'];
		$note   = $_REQUEST['note'];

		if ( 'earn' === $action ) {
			$user->earn_point( $amount, self::$hierarchy_wallet_id );
			$note = ! empty( $note ) ? $note : sprintf( __( '관리자에의해 %s포인트가 적립되었습니다.', 'mshop-point-ex' ), number_format( $amount, wc_get_price_decimals() ) );
			MSPS_Log::add_log( $_REQUEST['id'], msps_get_wallet_id( self::$hierarchy_wallet_id, null, $current_language ), 'earn', 'admin', $amount, $user->get_point(), 'completed', 0, $note );
		} else if ( 'deduct' == $action ) {
			$user->deduct_point( $amount, self::$hierarchy_wallet_id );
			$note = ! empty( $note ) ? $note : sprintf( __( '관리자에의해 %s포인트가 차감되었습니다.', 'mshop-point-ex' ), number_format( $amount, wc_get_price_decimals() ) );
			MSPS_Log::add_log( $_REQUEST['id'], msps_get_wallet_id( self::$hierarchy_wallet_id, null, $current_language ), 'deduct', 'admin', - 1 * $amount, $user->get_point(), 'completed', 0, $note );
		}

		wp_send_json_success();
	}

	/**
	 * @return void
	 */
	static function batch_adjust_hierarchy_point() {
		MSPS_Ajax::batch_adjust_point( self::$hierarchy_wallet_id );
	}
}


MSPS_Custom_Wallet_Manager::init();